@@ -15,9 +15,8 @@ from django.utils.safestring import mark_safe
 
 from .core import get_bootstrap_setting
 from .css import merge_css_classes
-from .exceptions import BootstrapError
-from .forms import WRAPPER_CLASS, WRAPPER_TAG, render_field, render_form, render_label
-from .size import DEFAULT_SIZE, SIZE_MD, SIZE_XS, get_size_class, parse_size
+from .forms import render_field, render_form, render_label
+from .size import DEFAULT_SIZE, SIZE_MD, get_size_class, parse_size
 from .text import text_value
 from .utils import render_template_file
 from .widgets import ReadOnlyPasswordHashWidget, is_widget_with_placeholder
@@ -28,7 +27,8 @@ class BaseRenderer(object):
 
     def __init__(self, *args, **kwargs):
         self.layout = kwargs.get("layout", "")
-        self.wrapper_class = kwargs.get("wrapper_class", WRAPPER_CLASS)
+        self.wrapper_class = kwargs.get("wrapper_class", get_bootstrap_setting("wrapper_class"))
+        self.inline_wrapper_class = kwargs.get("inline_wrapper_class", get_bootstrap_setting("inline_wrapper_class"))
         self.field_class = kwargs.get("field_class", "")
         self.label_class = kwargs.get("label_class", "")
         self.show_help = kwargs.get("show_help", True)
@@ -48,9 +48,12 @@ class BaseRenderer(object):
             "horizontal_field_offset_class", get_bootstrap_setting("horizontal_field_offset_class")
         )
         self.inline_field_class = kwargs.get("inline_field_class", get_bootstrap_setting("inline_field_class"))
+        self.server_side_validation = kwargs.get(
+            "server_side_validation", get_bootstrap_setting("server_side_validation")
+        )
         self.error_css_class = kwargs.get("error_css_class", None)
         self.required_css_class = kwargs.get("required_css_class", None)
-        self.bound_css_class = kwargs.get("bound_css_class", None)
+        self.success_css_class = kwargs.get("success_css_class", None)
         self.alert_error_type = kwargs.get("alert_error_type", "non_fields")
 
     @property
@@ -68,13 +71,6 @@ class BaseRenderer(object):
         """Return whether to render widgets with inline layout."""
         return self.layout == "inline"
 
-    def parse_size(self, size):
-        """Return size if it is valid, default size if size is empty, or throws exception."""
-        size = parse_size(size, default=DEFAULT_SIZE)
-        if size == SIZE_XS:
-            raise BootstrapError('Size "xs" is not valid for form controls.')
-        return size
-
     def get_size_class(self, prefix):
         """Return size class for given prefix."""
         return get_size_class(self.size, prefix=prefix) if self.size in ["sm", "lg"] else ""
@@ -97,16 +93,12 @@ class BaseRenderer(object):
             "checkbox_style": self.checkbox_style,
             "inline_field_class": self.inline_field_class,
             "error_css_class": self.error_css_class,
-            "bound_css_class": self.bound_css_class,
+            "success_css_class": self.success_css_class,
             "required_css_class": self.required_css_class,
             "alert_error_type": self.alert_error_type,
         }
         return context
 
-    def get_context_data(self):
-        """Return context data for rendering."""
-        return self.get_kwargs()
-
     def render(self):
         """Render to string."""
         return ""
@@ -117,15 +109,10 @@ class FormsetRenderer(BaseRenderer):
 
     def __init__(self, formset, *args, **kwargs):
         if not isinstance(formset, BaseFormSet):
-            raise BootstrapError('Parameter "formset" should contain a valid Django Formset.')
+            raise TypeError('Parameter "formset" should contain a valid Django Formset.')
         self.formset = formset
         super().__init__(*args, **kwargs)
 
-    def get_context_data(self):
-        context = super().get_context_data()
-        context["formset"] = self.formset
-        return context
-
     def render_management_form(self):
         """Return HTML for management form."""
         return text_value(self.formset.management_form)
@@ -162,15 +149,10 @@ class FormRenderer(BaseRenderer):
 
     def __init__(self, form, *args, **kwargs):
         if not isinstance(form, BaseForm):
-            raise BootstrapError('Parameter "form" should contain a valid Django Form.')
+            raise TypeError('Parameter "form" should contain a valid Django Form.')
         self.form = form
         super().__init__(*args, **kwargs)
 
-    def get_context_data(self):
-        context = super().get_context_data()
-        context["form"] = self.form
-        return context
-
     def render_fields(self):
         rendered_fields = mark_safe("")
         kwargs = self.get_kwargs()
@@ -213,7 +195,7 @@ class FieldRenderer(BaseRenderer):
 
     def __init__(self, field, *args, **kwargs):
         if not isinstance(field, BoundField):
-            raise BootstrapError('Parameter "field" should contain a valid Django BoundField.')
+            raise TypeError('Parameter "field" should contain a valid Django BoundField.')
         self.field = field
         super().__init__(*args, **kwargs)
 
@@ -251,11 +233,11 @@ class FieldRenderer(BaseRenderer):
         if self.field.form.empty_permitted:
             self.required_css_class = ""
 
-        bound_css_class = kwargs.get("bound_css_class", None)
+        success_css_class = kwargs.get("success_css_class", None)
         self.success_css_class = (
-            getattr(field.form, "bound_css_class", get_bootstrap_setting("success_css_class"))
-            if bound_css_class is None
-            else bound_css_class
+            getattr(field.form, "success_css_class", get_bootstrap_setting("success_css_class"))
+            if success_css_class is None
+            else success_css_class
         )
 
     @property
@@ -295,6 +277,10 @@ class FieldRenderer(BaseRenderer):
 
         return isinstance(widget, Textarea)
 
+    def can_widget_have_server_side_validation(self, widget):
+        """Return whether given widget can be rendered with server-side validation classes."""
+        return self.get_widget_input_type(widget) != "color"
+
     def can_widget_float(self, widget):
         """Return whether given widget can be set to `form-floating` behavior."""
         if self.is_form_control_widget(widget):
@@ -332,11 +318,8 @@ class FieldRenderer(BaseRenderer):
         if size_prefix:
             classes.append(get_size_class(self.size, prefix=size_prefix, skip=["xs", "md"]))
 
-        if self.field.errors:
-            if self.error_css_class:
-                classes.append(self.error_css_class)
-        elif self.field.form.is_bound:
-            classes.append(self.success_css_class)
+        if self.server_side_validation and self.can_widget_have_server_side_validation(widget):
+            classes.append(self.get_server_side_validation_classes())
 
         classes = before + classes
         widget.attrs["class"] = merge_css_classes(*classes)
@@ -428,6 +411,14 @@ class FieldRenderer(BaseRenderer):
             )
         return ""
 
+    def get_server_side_validation_classes(self):
+        """Return CSS classes for server-side validation."""
+        if self.field_errors:
+            return "is-invalid"
+        elif self.field.form.is_bound:
+            return "is-valid"
+        return ""
+
     def get_inline_field_class(self):
         """Return CSS class for inline field."""
         return self.inline_field_class or "col-12"
@@ -443,26 +434,28 @@ class FieldRenderer(BaseRenderer):
 
     def get_wrapper_classes(self):
         """Return classes for wrapper."""
-        wrapper_classes = [self.wrapper_class]
+        wrapper_classes = []
+
+        if self.is_inline:
+            wrapper_classes.append(self.get_inline_field_class())
+            wrapper_classes.append(self.inline_wrapper_class)
+        else:
+            if self.is_horizontal:
+                wrapper_classes.append("row")
+            wrapper_classes.append(self.wrapper_class)
 
         if self.is_floating:
             wrapper_classes.append("form-floating")
 
+        # The indicator classes are added to the wrapper class. Bootstrap 5 server-side validation classes
+        # are added to the fields
         if self.field.errors:
             wrapper_classes.append(self.error_css_class)
         elif self.field.form.is_bound:
             wrapper_classes.append(self.success_css_class)
-
         if self.field.field.required:
             wrapper_classes.append(self.required_css_class)
 
-        if self.is_inline:
-            wrapper_classes.append(self.get_inline_field_class())
-        else:
-            if self.is_horizontal:
-                wrapper_classes.append("row")
-            wrapper_classes.append("mb-3")
-
         return merge_css_classes(*wrapper_classes)
 
     def field_before_label(self):
@@ -471,11 +464,12 @@ class FieldRenderer(BaseRenderer):
 
     def render(self):
         if self.field.name in self.exclude.replace(" ", "").split(","):
-            return ""
+            return mark_safe("")
         if self.field.is_hidden:
             return text_value(self.field)
 
         field = self.get_field_html()
+
         if self.field_before_label():
             label = self.get_label_html()
             field = field + label
@@ -485,12 +479,8 @@ class FieldRenderer(BaseRenderer):
             label = self.get_label_html(horizontal=self.is_horizontal)
             horizontal_class = self.horizontal_field_class
 
-        if isinstance(self.widget, CheckboxInput):
-            field = format_html(
-                '<div class="{form_check_class}">{field}</div>',
-                form_check_class=self.get_checkbox_classes(),
-                field=field,
-            )
+        help = self.get_help_html()
+        errors = self.get_errors_html()
 
         if self.is_form_control_widget():
             addon_before = (
@@ -500,18 +490,27 @@ class FieldRenderer(BaseRenderer):
                 format_html('<span class="input-group-text">{}</span>', self.addon_after) if self.addon_after else ""
             )
             if addon_before or addon_after:
-                field = format_html('<div class="input-group mb-3">{}{}{}</div>', addon_before, field, addon_after)
+                classes = "input-group"
+                if self.server_side_validation and self.get_server_side_validation_classes():
+                    classes = merge_css_classes(classes, "has-validation")
+                field = format_html('<div class="{}">{}{}{}{}</div>', classes, addon_before, field, addon_after, errors)
+                errors = ""
+
+        if isinstance(self.widget, CheckboxInput):
+            field = format_html('<div class="{}">{}{}{}</div>', self.get_checkbox_classes(), field, errors, help)
+            errors = ""
+            help = ""
+
+        field_with_errors_and_help = format_html("{}{}{}", field, errors, help)
 
-        field_with_help_and_errors = format_html("{}{}{}", field, self.get_help_html(), self.get_errors_html())
         if self.is_horizontal:
-            field_with_help_and_errors = format_html(
-                '<div class="{}">{}</div>', horizontal_class, field_with_help_and_errors
+            field_with_errors_and_help = format_html(
+                '<div class="{}">{}</div>', horizontal_class, field_with_errors_and_help
             )
 
         return format_html(
-            '<{tag} class="{wrapper_classes}">{label}{field_with_help_and_errors}</{tag}>',
-            tag=WRAPPER_TAG,
+            '<div class="{wrapper_classes}">{label}{field_with_errors_and_help}</div>',
             wrapper_classes=self.get_wrapper_classes(),
             label=label,
-            field_with_help_and_errors=field_with_help_and_errors,
+            field_with_errors_and_help=field_with_errors_and_help,
         )
